function psd = powerspec(c, wintype, detrend, npad)
% function psd = powerspec(c, wintype, detrend, npad)

if nargin < 2,
   wintype = 'welch';
end;
if nargin < 3
   detrend = 1;
end
if nargin < 4
   npad = nan;
end

n = length(c);

if detrend
   c = c - c(1) - (c(n) - c(1)) * [0:(n-1)]' / (n-1);
   c = c - mean(c);
end

switch lower(wintype)
   case {'none'}
      win = ones(n,1);
   case {'bartlett'}
      win = 1.0 - abs( ([0:(n-1)]' - 0.5*n )/(0.5*n) );
   case {'welch'}
      win = 1.0 - ( ([0:(n-1)]' - 0.5*n )/(0.5*n) ).^2;
   case {'hann'}
      win = 0.5*( 1.0 - cos(2*pi*[0:(n-1)]'/n) );
   case {'bell'}
      win = ones(n,1); n4 = floor(n/4);
      win(1:n4) = 0.5*(1 - cos(([1:n4]'-1)*pi/n4));
      win(n-n4+1:n) = win(n4:-1:1);
   otherwise
      win = nan + zeros(n,1);
      disp('Unknown window');
end
wssi = 1.0 / (n*sum(win.^2));
c = c.*win;

if ~isnan(npad) & n < npad
   c = [ c ; zeros(npad-n,1) ];
end

fc = abs( fft(c) ).^2;

n2 = ceil(length(fc)/2);
psd = zeros(n2+1,1);

psd(1)    = wssi*fc(1);
psd(2:n2) = 2*wssi*fc(2:n2);
psd(n2+1) = wssi*fc(n2+1);

% [ var(c) sum(psd)*n/(n-1) ]


