function [time, p, Ta, RH, dir, spd, lon, lat, z] = ReadHSA(fname)
% function [time, p, Ta, RH, dir, spd, lon, lat, z] = ReadHSA(fname)
%
% Reads a dropsonde data file in hsa format from HFP98 website

f = fopen([fname '.hsa'], 'rt');
if (f < 0),
  display(['Can''t open ', fname]);
  return
end
time   = zeros(2000,1);
p      = zeros(2000,1); Ta     = zeros(2000,1); RH     = zeros(2000,1);
dir    = zeros(2000,1); spd    = zeros(2000,1); 
lon    = zeros(2000,1); lat    = zeros(2000,1); z      = zeros(2000,1);

buf = fgetl(f); count = 0;
while isstr(buf)
   if (length(buf) >= 1)
      rectype = sscanf(buf( 1: 2), '%d');
      if (rectype == 1)
         count = count + 1;
         day = sscanf(buf( 4:10),'%f');
         tim = sscanf(buf(12:15),'%f');
         time(count) = datenum(floor(day/10000) + 1900, ...
                               floor(mod(day,10000)/100), ...
                               mod(day,100), ...
                               floor(tim/10000), ...
                               floor(mod(tim,10000)/100), ...
                               mod(tim,100));
         lat(count)    = sscanf(buf( 17: 23),'%f');
         lon(count)    = sscanf(buf( 24: 31),'%f');
         p(count)      = sscanf(buf( 33: 38),'%f');
         Ta(count)     = sscanf(buf( 40: 45),'%f');
         RH(count)     = sscanf(buf( 47: 52),'%f');
         z(count)      = sscanf(buf( 54: 60),'%f');
         u             = sscanf(buf( 61: 66),'%f');
         v             = sscanf(buf( 68: 73),'%f');
         if feq(p(count), 1070)
            p(count) = z(count); z(count) = 0;
         end;
         if feq(u,-99) | feq(v,-99)
            dir(count) = nan;
            spd(count) = nan;
         else
            dir(count)    = mod( atan2(-u,-v)*180/pi, 360 );
            spd(count)    = hypot(u,v);
         end
      end
   end
   buf = fgets(f); 
end

time= time(1:count);
lat = lat(1:count); lat(find(feq(lat, -99))) = nan;
lon = lon(1:count); lon(find(feq(lon, -99))) = nan;
p   = p(  1:count); p(  find(feq(p  , -99))) = nan;
Ta  = Ta( 1:count); Ta( find(feq(Ta , -99))) = nan;
RH  = RH( 1:count); RH( find(feq(RH , -99))) = nan;
z   = z(  1:count); z(  find(feq(z  , -99))) = nan;
dir = dir(1:count); dir(find(feq(dir, -99))) = nan;
spd = spd(1:count); spd(find(feq(spd, -99))) = nan;


fclose(f);

