function [a,circ] = simplex_track(uv, xyt, a0, stormrel)
% function [a,circ] = simplex_track(uv, xyt, a0, stormrel)
% Given observations uv at xyt finds the track a = (dx,dy,ut,vt)' 
% which maximizes the circulation.

global simplex_uv simplex_xyt simplex_rel

if nargin <= 3
   stormrel = 1;
end

nobs = size(uv,1);

uc = 0; vc = 0;
for winditer = 0 : stormrel
   simplex_uv = uv - ones(nobs,1)*[uc vc]; 
   simplex_xyt = xyt; simplex_rel = 0;

   p = zeros(4,5);
   p(:,1) = a0;
   for ii = 2 : 5
      p(:,ii) = a0 + [1e3;1e3;1;1] .* randn(4,1);
   end
   [p fval nf] = amoebaf(p,1e-9,@circn,1000);
   a = p(:,1);
   uc = a(3); vc = a(4);
end
circ = -fval(1);
disp(sprintf('simplex_track: displacement = (%.2f %.2f) m, movement = (%.3f %.3f) m/s', ...
   a(1), a(2), a(3), a(4)));
disp(sprintf('simplex_track: circ=%.3f (%.3f), nf=%d', ...
   circ, -circn(a0), nf));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%5
function sumvc = circn(a)
% function sumvc = circn(a)
% Used with amoebaf to find cyclone centre and track by maximising the
% circulation for a set of observation.
% Returns the sum of the azimuthal wind components about the point (0,0) 
% for wind vectors uv(ii,:) at points xyt(ii,:)

global simplex_uv simplex_xyt simplex_rel

dx = a(1); dy = a(2); ut = a(3); vt = a(4);

xdx = simplex_xyt(:,1) - dx - ut*simplex_xyt(:,3); 
ydy = simplex_xyt(:,2) - dy - vt*simplex_xyt(:,3);
t   = simplex_xyt(:,3);
r = hypot(xdx, ydy);

if simplex_rel
   % Cartesian storm-relative wind components
   ux = simplex_uv(:,1) - ut; 
   vx = simplex_uv(:,2) - vt;  
else
   % Cartesian earth-relative wind components
   ux = simplex_uv(:,1); 
   vx = simplex_uv(:,2);  
end

% Project into cylindrical coordinates and sum
sumvc = -sum( (-ux.*ydy + vx.*xdx)./r );  

%if nargout >= 2
%   dvc = zeros( size(uv,1), 9 );
%   dvc(:,6) = vc.*xdx./r.^2 - vx./r;
%   dvc(:,7) = vc.*ydy./r.^2 + ux./r;
%   dvc(:,8) = t.*dvc(:,6);
%   dvc(:,9) = t.*dvc(:,7);
%end

