function [Tanal] = tcbarnes(xg,yg,Tb,xo,yo,To,Lrad,Laz,nits,kappa)
% function [Tanal] = tcbarnes(xg,yg,Tb,xo,yo,To,Lrad,Laz,nits,kappa)
%
% Does a 2-dimensional Barnes analysis of 
% observations To at locations (xo,yo) onto a grid (xg,yg) with background
% field Tb. (xg,yg) need to have been made using meshgrid and have constant
% spacing.
% Distances are defined in polar coordinates and scaled by Lrad and Laz.
% Does nits iterations with a rescaling of kappa each time.

if nargin == 0
   help tcbarnes
   Tanal = nan; 
   return
end

[ny,nx] = size(xg); nxy = nx*ny;
rg = hypot(xg,yg); thg = atan2(yg,xg); 
dx = xg(1,2) - xg(1,1); dy = yg(2,1) - yg(1,1);

ro = hypot(xo, yo); tho = atan2(yo,xo); no = length(xo);

if length(Tb) == 1
   Tb = repmat(Tb, size(xg));
end
Tanal = Tb;

for it = 1 : nits
   Tinc = To - interp2(xg,yg,Tanal,xo,yo);

   for ix = 1 : nx
      for iy = 1 : ny
         dth = abs( tho - thg(iy,ix) );
         ii = find( dth > pi );
         dth(ii) = 2*pi - dth(ii);
         dd = ( (ro - rg(iy,ix))/Lrad ).^2 + ( dth / Laz ).^2;   % squared distances
         wt = exp(-dd);
         wt = wt/sum(wt);
         Tanal(iy,ix) = Tb(iy,ix) + Tinc'*wt;
      end
   end

   %% Update length scales for next iteration
   Tb = Tanal;
   Lrad = Lrad*kappa; Laz = Laz*kappa;
end

