function [vc,dvc] = vaz_xy(uv, xy, a)
% Used with mrqmin_proj to fit analytical profiles to wind data.
% Returns the azimuthal wind component about the point (0,0) for wind vectors
% uv(ii,:) at points xy(ii,:). 

dx = a(6); dy = a(7);

xdx = xy(:,1) - dx; ydy = xy(:,2) - dy;
r = hypot(xdx, ydy);

ux = uv(:,1); vx = uv(:,2);

vc = (-ux.*ydy + vx.*xdx)./r;  % Project into cylindrical coordinates

dvc = zeros( size(uv,1), length(a) );
dvc(:,6) = vc.*xdx./r.^2 - vx./r;
dvc(:,7) = vc.*ydy./r.^2 + ux./r;


