function [xx,chisq] = wi_ch_asyn(xo,yo,to,uo,vo,wo,sigdir,relwind)
% function [xx,chisq] = wi_ch_asyn(xo,yo,to,uo,vo,wo,sigdir,relwind)
%     xx = [ xc yc uc vc ]

if nargin <= 5
   wo = ones(size(xo));
end

autowt = 0;
iter = 1;
if ischar(wo)
   if strcmpi(wo,'auto')
      if nargin <= 6
         disp('wi_ch_asyn: need sigdir argument');
         xx = nan; chisq = nan;
         return
      end
      autowt = 1;
      iter = 10;
      wo = ones(size(xo));
   else
      disp('wi_ch_asyn: Unrecognised character argument to wo');
      xc = nan; yc = nan;
      return;
   end
end

no = size(xo,1);
if size(yo) ~= [no 1] | size(uo) ~= [no 1] | size(vo) ~= [no 1] | size(wo) ~= [no 1]
   disp('wi_ch: argument size mismatch');
   xc = nan; yc = nan;
   return
end

uc = 0; vc = 0;
for winditer = 0 : relwind
   % Optional iteration loop to calculate and remove storm motion from winds.

   diro = atan2(-(uo - uc), -(vo - vc));

   sdir = sin(diro);
   cdir = cos(diro);
   term = xo.*sdir + yo.*cdir;

   for it = 1 : iter

      a11 = sum(wo.*sdir.^2);
      a12 = sum(wo.*sdir.*cdir);
      a13 = sum(wo.*to.*sdir.^2);
      a14 = sum(wo.*to.*sdir.*cdir);
      a22 = sum(wo.*cdir.^2);
      a23 = a14;
      a24 = sum(wo.*to.*cdir.^2);
      a33 = sum(wo.*(to.*sdir).^2);
      a34 = sum(wo.*(to.^2).*sdir.*cdir);
      a44 = sum(wo.*(to.*cdir).^2);
      
      b1 = sum(wo.*sdir.*term);
      b2 = sum(wo.*cdir.*term);
      b3 = sum(wo.*to.*sdir.*term);
      b4 = sum(wo.*to.*cdir.*term);
      
      A = [a11 a12 a13 a14 ; a12 a22 a23 a24 ; a13 a23 a33 a34 ; a14 a24 a34 a44];
      b = [b1 ; b2 ; b3 ; b4 ];
      
      rcon = rcond(A);
      if rcon > 4*eps
         xx = A\b;
      else  % Solve using SVD
         [u,s,v] = svd(A);
         diags = diag(s); diagsi = zeros(size(diags));
         thresh = no*eps*max(diags);
         ixin  = find(abs(diags) >  thresh);
         ixout = find(abs(diags) <= thresh);
         diagsi(ixin) = 1./diags(ixin);
         xx = v * diag(diagsi) * u' * b;
         if isempty(ixout)
            disp('mrqmin: SVD keeps all indices');
         else
            disp(sprintf('wi_ch_asyn: A badly conditioned, SVD keeps %d indep vars', ...
               length(ixin) ));
         end
      end
      
      xc = xx(1); yc = xx(2); uc = xx(3); vc = xx(4);

      delx = (xo - uc*to - xc);
      dely = (yo - vc*to - yc);
      
      chisq = sum( wo.*(delx.*sdir + dely.*cdir).^2 );

      if autowt
    %    wo = ( delx.*cdir - dely.*sdir ).^(-2);  % Tends to give erratic convergence
         sigs2 = sigdir.^2 .* max( delx.^2 + dely.^2, 3e3^2 );   % Eq 4 in paper
         wo = 1 ./ sigs2;
      end

   end  % Iteration loop to get weights.

end  % Iteration loop to get relative winds.

if 0
   fig(2); clf;
   subplot(3,1,1); cla reset;
   plot(hypot(delx,dely), wo, 'o');
   subplot(3,1,2); cla reset;
   plot(hypot(delx,dely), 1./sigdir, 'o');
   subplot(3,1,3); cla reset;
   plot(hypot(delx,dely), 1./max( delx.^2 + dely.^2, 5e3^2 ), 'o');
end
